import React, { useContext, useMemo, useRef, useEffect, useImperativeHandle, useCallback } from "react";
import { Text, TouchableOpacity, View, StyleSheet, Platform } from "react-native";
import Animated, { interpolate } from "react-native-reanimated";
import { addMonths, format, eachWeekOfInterval, eachDayOfInterval, addDays, lastDayOfMonth, isSameMonth, isSameDay, differenceInCalendarMonths } from "date-fns";
import { isEqual } from "lodash";
import InfinitePager from "react-native-infinite-pager";
const DEFAULT_FONT_FAMILY = Platform.OS === "ios" ? "Helvetica Neue" : "sans-serif";
const DEFAULT_FONT_COLOR_ACTIVE = "black";
const DEFAULT_FONT_COLOR_INACTIVE = "gray";
const DEFAULT_THEME = {
  todayIndicatorDotColor: "tomato",
  selectedDayBackgroundColor: "rgba(0, 0, 255, 0.25)",
  selectedDayFontColor: DEFAULT_FONT_COLOR_ACTIVE,
  headerFontFamily: DEFAULT_FONT_FAMILY,
  headerFontColor: DEFAULT_FONT_COLOR_ACTIVE,
  headerFontSize: 24,
  headerTextTransform: "capitalize",
  headerDateFormat: "MMMM yyyy",
  dayLabelFontFamily: DEFAULT_FONT_FAMILY,
  dayLabelColor: DEFAULT_FONT_COLOR_ACTIVE,
  dayLabelFontSize: 12,
  dayLabelTextTransform: "uppercase",
  dayLabelDateFormat: "EEEEEE",
  dayFontFamily: DEFAULT_FONT_FAMILY,
  dayFontColor: DEFAULT_FONT_COLOR_ACTIVE,
  dayInactiveFontColor: DEFAULT_FONT_COLOR_INACTIVE,
  daySelectedFontColor: DEFAULT_FONT_COLOR_ACTIVE,
  dayFontSize: 12,
  inactiveOpacity: 1
};
const CalendarContext = /*#__PURE__*/React.createContext({
  referenceDate: new Date(),
  selectedDate: null,
  onDateSelect: () => {},
  DayComponent: undefined,
  DayLabelComponent: undefined,
  HeaderComponent: undefined,
  theme: DEFAULT_THEME,
  pageInterpolator: defaultPageInterpolator
});
export function useCalendarContext() {
  return useContext(CalendarContext);
}

function defaultPageInterpolator({
  focusAnim,
  theme,
  pageWidth
}) {
  "worklet";

  return {
    transform: [{
      translateX: interpolate(focusAnim.value, [-1, 0, 1], [-pageWidth.value, 0, pageWidth.value])
    }],
    opacity: interpolate(focusAnim.value, [-1, 0, 1], [theme.inactiveOpacity, 1, theme.inactiveOpacity])
  };
}

export const MonthPage = /*#__PURE__*/React.memo(({
  index
}) => {
  const {
    referenceDate,
    HeaderComponent,
    DayLabelComponent,
    theme
  } = useCalendarContext();
  const firstDayOfMonth = useMemo(() => addMonths(referenceDate, index), [referenceDate, index]);
  firstDayOfMonth.setDate(1);
  const lastDayOfMo = useMemo(() => lastDayOfMonth(firstDayOfMonth), [firstDayOfMonth]);
  const headerText = format(firstDayOfMonth, theme.headerDateFormat);
  const weekStarts = useMemo(() => eachWeekOfInterval({
    start: firstDayOfMonth,
    end: lastDayOfMo
  }), [firstDayOfMonth, lastDayOfMo]);
  const weeks = useMemo(() => weekStarts.map(week => {
    return eachDayOfInterval({
      start: week,
      end: addDays(week, 6)
    });
  }), [weekStarts]);
  return /*#__PURE__*/React.createElement(View, null, HeaderComponent ? /*#__PURE__*/React.createElement(HeaderComponent, {
    date: firstDayOfMonth
  }) : /*#__PURE__*/React.createElement(View, {
    style: {
      alignItems: "center"
    }
  }, /*#__PURE__*/React.createElement(Text, {
    style: {
      fontSize: theme.headerFontSize,
      fontFamily: theme.headerFontFamily,
      color: theme.headerFontColor,
      textTransform: theme.headerTextTransform
    }
  }, headerText)), /*#__PURE__*/React.createElement(View, {
    style: styles.row
  }, /*#__PURE__*/React.createElement(View, {
    style: styles.dayLabelRow
  }, weeks[0].map(day => {
    const dayLabelText = format(day, theme.dayLabelDateFormat);
    return DayLabelComponent ? /*#__PURE__*/React.createElement(DayLabelComponent, {
      date: day
    }) : /*#__PURE__*/React.createElement(View, {
      key: `day-label-${day.toISOString()}`,
      style: styles.dayLabelContainer
    }, /*#__PURE__*/React.createElement(Text, {
      style: {
        color: theme.dayLabelColor,
        fontFamily: theme.dayLabelFontFamily,
        fontSize: theme.dayLabelFontSize,
        textTransform: theme.dayLabelTextTransform
      }
    }, dayLabelText));
  }))), /*#__PURE__*/React.createElement(Animated.View, {
    style: styles.row
  }, /*#__PURE__*/React.createElement(Animated.View, {
    style: {
      flex: 1
    }
  }, weeks.map(daysInWeek => {
    var _daysInWeek$;

    return /*#__PURE__*/React.createElement(Animated.View, {
      key: `week-${(_daysInWeek$ = daysInWeek[0]) === null || _daysInWeek$ === void 0 ? void 0 : _daysInWeek$.toISOString()}`,
      style: styles.weekContainer
    }, daysInWeek.map(day => {
      const sameMonth = isSameMonth(day, firstDayOfMonth);
      const dayDateFormatted = format(day, "yyyy-MM-dd");
      return /*#__PURE__*/React.createElement(DayWrapper, {
        key: dayDateFormatted,
        isInDisplayedMonth: sameMonth,
        date: day,
        dateFormatted: dayDateFormatted
      });
    }));
  }))));
});
const DayWrapper = /*#__PURE__*/React.memo(({
  date,
  isInDisplayedMonth,
  dateFormatted
}) => {
  const dateRef = useRef(date);
  const memoDate = useMemo(() => {
    if (isSameDay(dateRef.current, date)) {
      return dateRef.current;
    } else {
      dateRef.current = date;
      return date;
    }
  }, [date]);
  const {
    selectedDate,
    onDateSelect,
    DayComponent,
    theme
  } = useCalendarContext();
  const isSelected = useMemo(() => {
    return !!selectedDate && isSameDay(memoDate, selectedDate);
  }, [memoDate, selectedDate]);
  const isToday = useMemo(() => isSameDay(memoDate, new Date()), [memoDate]);
  const onDateSelectRef = useRef(onDateSelect);
  onDateSelectRef.current = onDateSelect;
  const onDateSelectCb = useCallback((date, options) => {
    var _onDateSelectRef$curr;

    return (_onDateSelectRef$curr = onDateSelectRef.current) === null || _onDateSelectRef$curr === void 0 ? void 0 : _onDateSelectRef$curr.call(onDateSelectRef, date, options);
  }, []);
  return /*#__PURE__*/React.createElement(DayItem, {
    date: memoDate,
    dateFormatted: dateFormatted,
    isSelected: isSelected,
    isToday: isToday,
    isInDisplayedMonth: isInDisplayedMonth,
    DayComponent: DayComponent,
    onDateSelect: onDateSelectCb,
    theme: theme
  });
});
const DayItem = /*#__PURE__*/React.memo(({
  date,
  isInDisplayedMonth,
  isSelected,
  DayComponent,
  isToday,
  onDateSelect,
  theme,
  dateFormatted
}) => {
  const dayText = format(date, "d");
  const deselectedColor = isInDisplayedMonth ? theme.dayFontColor : theme.dayInactiveFontColor;
  const color = isSelected ? theme.daySelectedFontColor : deselectedColor;

  if (DayComponent) {
    return /*#__PURE__*/React.createElement(DayComponent, {
      date: date,
      isInDisplayedMonth: isInDisplayedMonth,
      isSelected: isSelected,
      isToday: isToday
    });
  }

  const padding = 10;
  return /*#__PURE__*/React.createElement(TouchableOpacity, {
    testID: `react-native-swipe-calendar:${dateFormatted}`,
    onPress: () => onDateSelect === null || onDateSelect === void 0 ? void 0 : onDateSelect(date, {
      isSelected
    }),
    style: {
      flex: 1,
      justifyContent: "center",
      alignItems: "center",
      flexDirection: "row",
      padding
    }
  }, /*#__PURE__*/React.createElement(View, {
    style: {
      flex: 0,
      aspectRatio: 1,
      position: "absolute",
      top: 0,
      bottom: 0,
      backgroundColor: isSelected ? theme.selectedDayBackgroundColor : "transparent",
      borderRadius: 5
    }
  }), /*#__PURE__*/React.createElement(Text, {
    style: {
      color,
      fontSize: theme.dayFontSize,
      fontFamily: theme.dayFontFamily
    }
  }, dayText), /*#__PURE__*/React.createElement(View, {
    style: {
      position: "absolute",
      width: 5,
      height: 5,
      bottom: padding / 2,
      borderRadius: 5,
      backgroundColor: isToday ? theme.todayIndicatorDotColor : "transparent"
    }
  }));
});

function Calendar({
  selectedDate,
  onDateSelect,
  onMonthChange,
  currentDate,
  HeaderComponent,
  DayLabelComponent,
  DayComponent,
  theme = {},
  monthBuffer = 1,
  minDate,
  maxDate,
  pageInterpolator = defaultPageInterpolator,
  simultaneousHandlers
}, ref) {
  const initialDateRef = useRef(currentDate || new Date());
  const pagerRef = useRef(null);
  const currentDateRef = useRef(currentDate);
  const currentPageRef = useRef(0);
  const minPageIndex = useMemo(() => {
    if (!minDate) return -Infinity;
    return differenceInCalendarMonths(initialDateRef.current, minDate) * -1;
  }, [minDate]);
  const maxPageIndex = useMemo(() => {
    if (!maxDate) return Infinity;
    return differenceInCalendarMonths(initialDateRef.current, maxDate) * -1;
  }, [maxDate]);
  const onMonthChangeRef = useRef(onMonthChange);
  onMonthChangeRef.current = onMonthChange;
  const fullThemeObj = { ...DEFAULT_THEME,
    ...theme
  };
  const fullThemeRef = useRef(fullThemeObj);
  const fullTheme = useMemo(() => {
    const updatedTheme = { ...DEFAULT_THEME,
      ...theme
    };

    if (isEqual(fullThemeRef.current, updatedTheme)) {
      return fullThemeRef.current;
    } else {
      fullThemeRef.current = updatedTheme;
      return updatedTheme;
    }
  }, [theme]);
  useImperativeHandle(ref, () => ({
    incrementMonth: options => {
      var _options$animated, _pagerRef$current;

      const animated = (_options$animated = options === null || options === void 0 ? void 0 : options.animated) !== null && _options$animated !== void 0 ? _options$animated : true;
      (_pagerRef$current = pagerRef.current) === null || _pagerRef$current === void 0 ? void 0 : _pagerRef$current.incrementPage({
        animated
      });
    },
    decrementMonth: options => {
      var _options$animated2, _pagerRef$current2;

      const animated = (_options$animated2 = options === null || options === void 0 ? void 0 : options.animated) !== null && _options$animated2 !== void 0 ? _options$animated2 : true;
      (_pagerRef$current2 = pagerRef.current) === null || _pagerRef$current2 === void 0 ? void 0 : _pagerRef$current2.decrementPage({
        animated
      });
    },
    setMonth: (date, options) => {
      var _options$animated3, _pagerRef$current3;

      const animated = (_options$animated3 = options === null || options === void 0 ? void 0 : options.animated) !== null && _options$animated3 !== void 0 ? _options$animated3 : false;
      const page = differenceInCalendarMonths(date, initialDateRef.current);
      (_pagerRef$current3 = pagerRef.current) === null || _pagerRef$current3 === void 0 ? void 0 : _pagerRef$current3.setPage(page, {
        animated
      });
    }
  }), []);
  useEffect(() => {
    if (currentDate && currentDateRef.current && !isSameMonth(currentDate, currentDateRef.current)) {
      var _pagerRef$current4;

      const page = differenceInCalendarMonths(currentDate, initialDateRef.current);
      if (page === currentPageRef.current) return;
      (_pagerRef$current4 = pagerRef.current) === null || _pagerRef$current4 === void 0 ? void 0 : _pagerRef$current4.setPage(page, {
        animated: false
      });
    }

    currentDateRef.current = currentDate;
  }, [currentDate]);
  const onPageChange = useCallback(pg => {
    var _onMonthChangeRef$cur;

    currentPageRef.current = pg;
    const currentMonth = addMonths(initialDateRef.current, pg);
    currentMonth.setDate(1);
    (_onMonthChangeRef$cur = onMonthChangeRef.current) === null || _onMonthChangeRef$cur === void 0 ? void 0 : _onMonthChangeRef$cur.call(onMonthChangeRef, currentMonth);
  }, []);
  const providerValue = useMemo(() => ({
    referenceDate: initialDateRef.current,
    selectedDate,
    onDateSelect,
    HeaderComponent,
    DayLabelComponent,
    DayComponent,
    theme: fullTheme,
    pageInterpolator
  }), [selectedDate, onDateSelect, HeaderComponent, DayLabelComponent, DayComponent, fullTheme, pageInterpolator]);
  const pageInterpolatorInternal = useCallback(params => {
    "worklet";

    return pageInterpolator(Object.assign(params, {
      theme: fullTheme
    }));
  }, [fullTheme, pageInterpolator]);
  return /*#__PURE__*/React.createElement(CalendarContext.Provider, {
    value: providerValue
  }, /*#__PURE__*/React.createElement(InfinitePager, {
    ref: pagerRef,
    PageComponent: MonthPage,
    pageBuffer: monthBuffer,
    onPageChange: onPageChange,
    minIndex: minPageIndex,
    maxIndex: maxPageIndex,
    pageInterpolator: pageInterpolatorInternal,
    simultaneousHandlers: simultaneousHandlers
  }));
}

export default /*#__PURE__*/React.memo( /*#__PURE__*/React.forwardRef(Calendar));
const styles = StyleSheet.create({
  flex: {
    flex: 1
  },
  row: {
    flexDirection: "row"
  },
  dayLabelContainer: {
    flex: 1,
    justifyContent: "center",
    alignItems: "center"
  },
  weekContainer: {
    flexDirection: "row",
    justifyContent: "space-between"
  },
  dayLabelRow: {
    flex: 1,
    flexDirection: "row",
    justifyContent: "space-between",
    paddingVertical: 10
  }
});
//# sourceMappingURL=index.js.map